		/*  STP PROGRAMMATIC INTERFACE CLIENT   */

/*  Programmatic interface client for the Seismic Transfer Program
 *  N. Mullaney, Caltech, Nov, 2003  
 *
 *  Based on :Client side of the Seismic Transfer Program
 *  R. Clayton, Caltech, Sept, 2000
 *  
 *  
 */

/*
 * On Solaris compile as:
 *	cc -o stp stp.c -lnsl -lsocket
 * On Linux compile as:
 *	cc -o stp stp.c -lresolv 
 */

#include "stp_int.h"
#include <stdlib.h>

char databuf[DATABUFSZ];
int verbose     = 0;
int nrowset     = 24;
char masterdir[256] = ".";
FILE *ws_fdr, *ws_fdw;
int ws_sockfd;
int iserv;

int STR_FLAG=0;
int FILE_FLAG=0;
int DATA_FLAG=0;
int DIR_FLAG=0;
int ERR_FLAG=0;


/* start_stp_request opens the stp connection and sends a command to
 * the stp server.  
 */
void
send_stp_request(char line1[MAXLINE])
{
   char com[64], line[MAXLINE];
   // we need to add an endline to the request
   sprintf(line,"%s\n",line1);
   ws_putline(line);
   return;
}


/* 
 * read a string from the server
 *
 */
int
stp_string_read(char data[MAXLINE])
{
  
  char line[MAXLINE], com[64];
  int return_val=0;
  // STR_FLAG == 0 could mean that we're out of data
  // or that this is the first time stp_string_read
  // has been called since the request
  if (STR_FLAG == 0) {
      ws_getline(line, MAXLINE);
      sscanf(line, "%s", com);
      if (strcmp(com, "MESS") == 0)
         STR_FLAG=1;
      else if (strcmp(com, "ERR") == 0)
         ERR_FLAG=1;
      else {
         strcpy(data, "No more data.");
         return_val = 0;
      }
  }
  // If there is a string waiting, the string is put in data
  if (STR_FLAG == 1) {
    if (ws_getline(line,MAXLINE) != EOF) {
      sscanf(line,"%s",com);
      if(strcmp("ENDmess", com) == 0 ||
	 strcmp("OVER", com) == 0) {
	// get rid of OVER after ENDmess
        // set return_val and STR_FLAG to reflect
        // that there is no more data
	STR_FLAG = 0;
	return_val =  0; 
        strcpy(data, "No data.");
      }
      else {
	strcpy(data, line);
	return_val = 1;
      }
    }
    // if we got EOF, set STR_FLAG and return_val so to
    // show that there is no more data
    else {
      strcpy(data, "No data.");
      STR_FLAG = 0;
      return_val = 0;
    }
  }
  else if (ERR_FLAG == 1) {
    sprintf(data, "Error message: %s", &line[3]);
    ERR_FLAG = 0;
    return_val = -1;
  }

  return return_val;
}

/*
 * read the name of a file from the server
 *
 */
int stp_file_read(char *filename)
{
  char line[MAXLINE], com[64];
  int return_val = 0;
  if (ws_getline(line, MAXLINE) != EOF) {
     sscanf(line,"%s",com);
     if (strcmp(com, "OVER") == 0) {
        strcpy(filename, "No data (OVER)");
        return_val = 0;
     }
     else if (strcmp(com, "FILE") != 0) {
        strcpy(filename, "not_a_file");
        return_val = -1;
     }
     else {
        sscanf(line, "%s %s", com, filename);
        return_val = 1;
     }
  }
  else {
    strcpy(filename, "No data (EOF).");
    return_val = 0;
  }

  return return_val;
}

/*
 * read the name of the directory from the server
 *
 */
int stp_dir_read(char *dir)
{
  char line[MAXLINE], com[64];
  int return_val = 0;
  if (ws_getline(line, MAXLINE) != EOF) {
     sscanf(line,"%s %s",com,dir);
     if (strcmp(com, "OVER") == 0) {
        return_val = 0;
        strcpy(dir, "No data.");
     }
     else if (strcmp(com, "DIR") != 0) {
        return_val = -1;
        strcpy(dir, "not_a_dir");
     }
     else
        return_val = 1;
  }
  else {
    return_val = 0;
    strcpy(dir, "EOF");
  }

  return return_val;
}

// Since the amount of data returned by a data read
// is variable, this function will return how much 
// space is needed for the next data read.
// MUST BE DONE before stp_data_read!!!
int stp_data_prep() {
  char line[MAXLINE], com[64];
  int return_val = 0;
  int ndata;

  ws_getline(line, MAXLINE);
  sscanf(line, "%s", com);
  if (strcmp(com, "DATA") == 0) {
    sscanf(line, "%s %d", com, &DATA_FLAG);
    return_val = DATA_FLAG;
  }
 
  return return_val;
}

/*
 * read data from the server into the buffer
 * 
 * returns the amount of data read or -1 for error
 */  
int stp_data_read(char *buf) {
  char com[64], line[MAXLINE];
  int ndatahold, n;
  n = ws_getdata(buf, DATA_FLAG);
  if (n != DATA_FLAG) {
    sprintf(buf, "read error from server\n");
    return -1;
  }
  DATA_FLAG=0;
  ws_getline(line, MAXLINE);
  sscanf(line, "%s", com);
  if (strcmp(com, "ENDdata") != 0) {
    sprintf(buf, "bad read from server\n");
    return -1;
  }
  return n;
}


/*
 * opens the waveserver
 */
void
open_waveserver(char *server_name)
   {
	int two, temp, c;
	FILE *fdopen();
	struct sockaddr_in servaddr, cliaddr;
	struct hostent *hp;
	struct in_addr **pptr;
	char line[64], in[MAXLINE];
	/*socklen_t len;*/
	uint len;

	/* see if caller if fussy about which server */
	if(strcmp(server_name,"*") == 0)
	   {
		for(iserv= 0; address[iserv].name[0] != '\0'; iserv++)
		   {
			/* get IP number of stp server */
			if( (hp= gethostbyname(address[iserv].addr)) == NULL )
			   {
				err(WARN,"STP: gethostbyname error %s\n",address[iserv].addr);
				continue;
			   }
			pptr= (struct in_addr **)hp->h_addr_list;
			/* set up a socket */
			if ( (ws_sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
				err(WARN,"Wave_Server: socket connect error\n");
			/* connect socket to Wave Server Port */
			bzero(&servaddr, sizeof(servaddr));
			servaddr.sin_family = AF_INET;
			servaddr.sin_port   = htons(address[iserv].port);
			memcpy(&servaddr.sin_addr,*pptr,sizeof(struct in_addr));
		
			if (connect(ws_sockfd, (SA *) &servaddr, sizeof(servaddr)) >= 0) break;
				
		   }
		if(address[iserv].name[0] == 0)
			err(FATAL,"STP: connect error\n");
	   }
	  else
	   {
		/* find server in table */
		for(iserv= 0; address[iserv].name[0] != '\0'; iserv++)
			if(strcmp(address[iserv].name,server_name) == 0) break;
		if(address[iserv].name[0] == '\0')
			err(FATAL,"Unknown server = %s\n",server_name);

		/* get IP number of stp server */
		if( (hp= gethostbyname(address[iserv].addr)) == NULL )
			err(FATAL,"STP: gethostbyname error\n");
		pptr= (struct in_addr **)hp->h_addr_list;
		/* set up a socket */
		if ( (ws_sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
			err(FATAL,"Wave_Server: socket connect error\n");
		/* connect socket to Wave Server Port */
		bzero(&servaddr, sizeof(servaddr));
		servaddr.sin_family = AF_INET;
		servaddr.sin_port   = htons(address[iserv].port);
		memcpy(&servaddr.sin_addr,*pptr,sizeof(struct in_addr));
		if (connect(ws_sockfd, (SA *) &servaddr, sizeof(servaddr)) < 0)
			err(FATAL,"STP: connect error\n");
	   }
	len = sizeof(cliaddr);
	if (getsockname(ws_sockfd, (SA *) &cliaddr, (int *)(&len)) < 0)
		err(WARN,"STP: getsockname error");
	
	ws_fdr= fdopen(ws_sockfd,"r");
	ws_fdw= fdopen(ws_sockfd,"w");
	if( ws_fdr == NULL || ws_fdw == NULL)
		err(FATAL,"STP: cannot open ws_fdw/ws_fdr\n");
	
	/* send authorization */
	sprintf(line,"%s %s %s %s\n","STP",PASSWD,VERSION, "pgi");
	ws_putline(line);
	fflush(ws_fdw);

	/* get connection acknowledgement */
	ws_getline(line,MAXLINE);
	if(strcmp(line,"CONNECTED\n") != 0)
		err(FATAL,"STP: Connection rejected by waveserver - %s\n", line);

	//fprintf(stderr,"STP: Connected to %s\n",address[iserv].addr);

	/* send a binary sample so the server can establish byte order */
	two= 2;
	ws_putdata(&two,4);
	fflush(ws_fdw);

	/* receive possible message from server but do nothing with it*/
	receive_data();
        
   }

void
receive_data()
   {
	char line[MAXLINE+1], com[MAXLINE+1];
		
	while( ws_getline(line,MAXLINE) != EOF)
	   {
		sscanf(line,"%s",com);

		if(strcmp("OVER",com) == 0)
		   {
		     
		     return;
		   }
		if(strcmp("MESS",com) == 0)
		   {
                        
			int cnt, nrow;
			char buf[24];
			nrow=nrowset;
			cnt= 0;
			/* transfer verbatim messages */
			while( ws_getline(line,MAXLINE) != EOF)
			   {
			     sscanf(line,"%s",com);
			     if(strcmp("ENDmess",com) == 0) break;
			     cnt++;
			     if(cnt%nrow == 0)
			       {
				 fgets(buf,32,stdin);
				 if(buf[0] >= '0' && buf[0] <= '9') nrow= atol(buf);
				 if(buf[0] == 'q' || buf[0] == 'Q')
				   {
				     /* suck the remaining out of the input */
				     while( ws_getline(line,MAXLINE) != EOF)
				       {
					 sscanf(line,"%s",com);
					 if(strcmp("ENDmess",com) == 0) break;
				       }
				     break;
				   }
			       }
			     
			   }
			continue;
		   }
		if(strcmp("ERR",com) == 0)
		   {
                        printf("ERR\n");
			/* transfer 1-line error message */
			fprintf(stderr,"Error Message;\n");
			fprintf(stderr,"%s",&line[3]);
			continue;
		   }

	   }
   }

void
close_waveserver()
   {
	fclose(ws_fdr);
	fclose(ws_fdw);
	close(ws_sockfd);
   }

void
ws_putline(char *str)
   {
	fwrite(str,1,strlen(str),ws_fdw);
	fflush(ws_fdw);
   }

int
ws_getline(char *str, int maxlen)
   {
	char *p, *fgets();
	p= fgets(str,maxlen,ws_fdr);
	if(p == NULL) return(EOF);
	return(strlen(p));
   }

int
ws_getdata(int *buf, int ndata)
   {
	int n;
	n= fread(buf,1,ndata,ws_fdr);
	return(n);
   }

int
ws_putdata(int *buf, int ndata)
   {
	int n;
	n= fwrite(buf,1,ndata,ws_fdw);
	return(n);
   }


err(int mode, char *fmt, int a1, int a2, int a3, int a4)
   {
	fprintf(stderr,fmt,a1,a2,a3,a4);
	if(mode == FATAL) exit(-1);
   }



