#include "sac.h"
#include <assert.h>
#include <stdlib.h>

#define SAC_HEAD_SIZE sizeof(struct sac_header)

/* buf_to_sac_input takes a char buffer given by
 * stp_data_read in the SAC format and moves the
 * data into a sacinput struct
 */
void buf_to_sac_input(char *buf, size_t buf_size, struct sac_header *header, float *fdata) {
   memcpy(header, &sac_null, SAC_HEAD_SIZE);
   memcpy(header, buf, SAC_HEAD_SIZE);
   buf+= SAC_HEAD_SIZE;
   memcpy(fdata, buf, (buf_size - SAC_HEAD_SIZE));
   return;
   }

/* stp_sacinput_read combines the steps of reading data
 * into the buffer and moving it into the sac format
 */
int stp_sacinput_read(struct sac_header *header, float *fdata, size_t bytes) {
   char *buf;
   int n;
   buf = (char *) malloc(bytes);
   if (buf == NULL) {
     return -1;
   }
   n = stp_data_read(buf); 
   buf_to_sac_input(buf, bytes, header, fdata);
   free(buf);  
   return n;
}

/* converts the data from the sac_header to a human-readable string
 */
int sac_header_to_string(char *string, struct sac_header *sac) {
  char temp[100];

  /* here's the stuff that needs to go in:
     sac.depmin	= dmin;
     sac.depmax	= dmax;
     sac.depmen	= dmean;
     
     // maybe, check for -12345
     sac.evla	= ev->lat;
     sac.evlo	= ev->lon;
     sac.evdp	= ev->depth;
     
     sac->npts = ndata; 
     sac->delta = seis->dt; 
     sac->b = seis->t1 - t; 
     sac->e = sac->b +seis->t2 - seis->t1; 
     sac->o = 0.0;
     sac->iftype = ITIME; 
     sac->idep = IUNKN; 
     sac->idep= IDISP; 
     sac->idep=IVEL; 
     sac->idep= IACC; 
     sac->leven =  TRUE; 
     sac->lpspol = FALSE; 
     sac->lcalda = TRUE; 
     sac->internal4= 6; 
     strcpy(sac->knetwk,seis->nscl.net);
     strcpy(sac->kstnm,seis->nscl.sta);
     strcpy(sac->kinst,seis->nscl.chan);
     strcpy(sac->kcmpnm,seis->nscl.chan); 
     sac->stla =seis->sta_lat; 
     sac->stlo = seis->sta_lon; 
     sac->stel =seis->sta_elev;
	
     sac->stdp = 90.0 - seis->sta_dip; sac->cmpaz = seis->sta_az;
     sac->cmpinc = 0.0;
     
     sac->cmpinc = 90.0 - seis->sta_dip;
     
     
     sac->nzyear = ptm->tm_year + 1900; 
     sac->nzjday = ptm->tm_yday + 1; 
     sac->nzhour = ptm->tm_hour; 
     sac->nzmin = ptm->tm_min;
     sac->nzsec = ptm->tm_sec; 
     sac->nzmsec = (int)(msec*1000.0 + 0.5);
     
   */
  assert(string != NULL);
  strcpy(string, "");
  strcat(string, sac->knetwk);
  strcat(string, " ");
  strcat(string, sac->kstnm);
  strcat(string, " ");
  strcat(string, sac->kinst);
  strcat(string, " ");
  sprintf(temp, "%f", sac->stla);
  strcat(string, temp);
  strcat(string, " ");
  sprintf(temp, "%f", sac->stlo);
  strcat(string, temp);
  strcat(string, " ");
  sprintf(temp, "%f", sac->stel);
  strcat(string, temp);
  strcat(string, " ");
  sprintf(temp, "%d", sac->npts);
  strcat(string, temp);

  return 0;
}
