/*
 *  Test file for stp_int
 *
 */

#include "sac.h"
#include "fmtX.h"

#include	<stdio.h>
#include	<time.h>
#include	<sys/types.h>	/* basic system data types */
#include	<sys/socket.h>	/* basic socket definitions */
#include	<netinet/in.h>	/* sockaddr_in{} and other Internet defns */
#include	<arpa/inet.h>	/* inet(3) functions */
#include	<errno.h>
#include	<fcntl.h>		/* for nonblocking */
#include	<netdb.h>
#include	<signal.h>
#include	<sys/un.h>
#include        <stdlib.h>

#define SAC_HEAD_SIZE sizeof(struct sac_header)
#define MAXLINE 256

int test_ascii();
int test_string();
int test_win();
int test_trig();
int test_fmtX();

int
main()
{

  //  test_ascii();
  
  //  test_string();
  
  //  test_win();
  
  test_trig();

  //  test_fmtX();

  exit(0);
}

int test_ascii() {
  char event[MAXLINE], dir[MAXLINE], file[MAXLINE];
  int ndata, data_size, i;
  char *buf;
  FILE *out;
  char *temp;
  float *data;

  open_waveserver("*");
  send_stp_request("ascii");
  stp_string_read(event);
  
  send_stp_request("trig -net CI -sta PAS -chan BHZ 9818433");
  //9818433 13938180
  
  stp_dir_read(dir);
  printf("dir : %s \n", dir);
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep();
  buf = (char *) malloc(ndata);
  if (buf == NULL) {
    return -1;
  }
  stp_data_read(buf);
  stp_file_read(file);
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep();
  buf = (char *) malloc(ndata);
  if (buf == NULL) {
    return -1;
  }
  stp_data_read(buf);
  
  out = fopen(file, "w");
  fwrite(buf, 1, ndata, out);
  fclose(out);
  close_waveserver();
  
  temp = (char *) malloc(70 * sizeof(char));
  if (temp == NULL) {
    return -1;
  }
  data = (float *) malloc(ndata);
  if (data == NULL) {
    return -1;
  }
  
  data_size = buf_to_ascii(buf, ndata, temp, data);

  printf("temp\n%s\n", temp);
  printf("data_size = %d\n", data_size);
  for (i = 0; i< data_size; i++) {
    printf("data[%d] = %12.5e\n", i, data[i]);
  }
  
  
  free(temp);
  free(data); 
  free(buf);
  return 0;
}

int test_string() {
  char event[MAXLINE];
  int in;

  open_waveserver("*");
  send_stp_request("evt -e 13999108");

  in = stp_string_read(event);
 
  while(in == 1)
    {
      printf("%s\n", event);
      in = stp_string_read(event);
    }
  if (in == -1)
    printf("error: %s\n", event);

  close_waveserver();
  printf("next event : \n");
  open_waveserver("*");
  send_stp_request("event -e 1399920");
  in = stp_string_read(event);
  while(in == 1)
    {
      printf("%s\n", event);
      in = stp_string_read(event);
    }
   if (in == -1)
    printf("error: %s\n", event);

  close_waveserver();
  return 0;
}

int test_win() {
  char file[MAXLINE];
  int ndata;
  float *fdata;
  struct sac_header *header;
 
  // testing win
  open_waveserver("*");
  send_stp_request("win CI PAS BHZ 2003/08/27,06:01:00 +1m");  
      
  while(stp_file_read(file) == 1) {
    ndata = stp_data_prep();
    header = (struct sac_header *) malloc(SAC_HEAD_SIZE);
    if (header == NULL) {
      return -1;
    }
    fdata = (float *) malloc(ndata - SAC_HEAD_SIZE);
    if (fdata == NULL) {
      return -1;
    }
    stp_sacinput_read(header, fdata, ndata);
    
    printf("should be 6 : %d\n", header->internal4);
    printf("delta : %f\n", header->delta);
    printf("depmin : %f\n", header->depmin);
    printf("depmax : %f\n", header->depmax);
    printf(" sta: %s chan: %s \n", header->kstnm, header->kinst);
    free(header);
    free(fdata);
  }
  close_waveserver();
  return 0;
}

int test_trig(){
  char dir[MAXLINE], file[MAXLINE];
  int ndata;
  struct sac_header *header;
  float *fdata;
  char *info;
  
  //testing trig
  open_waveserver("*");
  send_stp_request("trig -net CI -sta PAS -chan EHZ 9818433");
  stp_dir_read(dir);
  printf("dir: %s\n", dir);     
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep();
  info = (unsigned char *)malloc(ndata+1);
  if (info == NULL) {
    return -1;
  }
  printf("ndata = %d\n", ndata);  
  stp_data_read(info);
  info[ndata] = '\0';
  printf("info : %s \n", info);
  stp_file_read(file);
  printf("file: %s \n", file);
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep();
  printf("ndata = %d\n", ndata);
  header = (struct sac_header *) malloc(SAC_HEAD_SIZE);
  if (header == NULL) {
    return -1;
  }
  fdata = (float *) malloc(ndata - SAC_HEAD_SIZE);
  if (fdata == NULL) {
    return -1;
  }
  stp_sacinput_read(header, fdata, ndata);
  
  printf("should be 6 : %d\n", header->internal4);
  printf("delta : %f\n", header->delta);
  printf("depmin : %f\n", header->depmin);
  printf("depmax : %f\n", header->depmax);
  printf("scale : %f\n", header->scale);
  printf(" sta: %s chan: %s \n", header->kstnm, header->kinst);
  printf(" net: %s \n", header->knetwk);
  printf(" npts : %d \n", header->npts);

  free(info);
  free(header);
  free(fdata);
  
  close_waveserver();
  return 0;
}

int test_fmtX() {
  char dir[MAXLINE], file[MAXLINE], temp[MAXLINE];
  int ndata;
  struct Xhdr *header;
  float *fdata;
  char *info;

  open_waveserver("*");
  send_stp_request("xfmt");
  stp_string_read(temp);
  send_stp_request("trig -net CI -sta PAS -chan BHZ 9818433");
  stp_dir_read(dir);
  printf("dir: %s\n", dir);     
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep();
  info = (unsigned char *)malloc(ndata+1);
  if (info == NULL) {
    return -1;
  }
  printf("ndata = %d\n", ndata);  
  stp_data_read(info);
  info[ndata] = '\0';
  printf("info : %s \n", info);
  stp_file_read(file);
  printf("file: %s \n", file);
  stp_file_read(file);
  printf("file: %s \n", file);
  ndata = stp_data_prep(); 
  header = (struct Xhdr *) malloc(sizeof(struct Xhdr));
  if (header == NULL) {
    return -1;
  }
  fdata = (float *) malloc(ndata - sizeof(struct Xhdr));
  if (fdata == NULL) {
    return -1;
  }
  stp_fmtX_read(header, fdata, ndata);

  // test the header
  printf("version = %d\n", header->version);
  printf("net = %s\n", header->net);
  printf("sta = %s\n", header->sta);
  printf("chan = %s\n", header->chan);
  printf("tstart = %f\n", header->tstart);

  free(info);
  free(fdata);
  free(header);

  close_waveserver();
  return 0;
}
