/*
 * example file used to test stp_user_int.c
 *
 */

#include <string.h>
#include "sac.h"
#include "stp_user_int.h"

void test_win();
void test_trig();
void test_event();
void test_altloc();
void test_sta();
void test_phase();


int main() {

  printf("Testing win\n");
  test_win();
  printf("Testing trig\n");
  test_trig();
  printf("Testing event\n");
  test_event();
  printf("Testing altloc\n");
  test_altloc();
  printf("Testing sta\n");
  test_sta();
  printf("Testing phase\n");
  test_phase();

  return 0;
}


/* test_win requests continuous waveforms from the stp server
 * and gets them in SAC format
 */ 
void test_win() {

  int n, c;
  char *chead;
  float **data;
  struct sac_header *shead;
  
  /* allocate data */
  chead = (char *)malloc(632 * sizeof(char));
  shead = (struct sac_header *)malloc(sizeof(struct sac_header));
  data = (float **)malloc(sizeof(float));
  *data = NULL;
 
  /* request a win */  
  stp_set_defaults("set fill 0; set nevntmax 20");
  n = stp_req_waveform("win CI PAS BH_ 2003/08/27,06:01:00 +1m");

  /* if there is no error */
  if (n != -1) {    
    /* get the waveforms */
    while((n = stp_get_sac_waveform (chead, shead, data, 99999)) > 0) {
      printf("%s\n", chead);
      /* print out some information from the header */
      printf("got %s.%s.%s npts=%d\n", shead->knetwk, shead->kstnm, shead->kcmpnm, (n - 632)/4);

    }
    
    if(n < 0) {
      printf("Error: %s\n", chead);
    }
    else if (n==0) {
      printf("no more files\n");
    }
  }
  else
    printf("Request failed\n");
  
  /* free up allocated data */
  free(chead);
  free(shead);
  free(*data);
  free(data);
}


/* requests waveforms for a triggered event and gets them */
void test_trig() {
  int n, c;
  char *chead;
  float **data;
  struct sac_header *shead;

  /* allocate data */
  chead = (char *)malloc(632 * sizeof(char));
  strcpy(chead, "nothing");
  shead = (struct sac_header *)malloc(sizeof(struct sac_header));
  data = (float **)malloc(sizeof(float));
  *data = NULL;

  /* make a request to the server */
  n = stp_req_waveform("trig -net CI -sta PAS -chan BHZ 9818433");

  /* if there is no error */
  if (n != -1) {
    
    /* get waveforms */
    while((n = stp_get_sac_waveform (chead, shead, data, 100)) > 0) {
      /* print out parts of the header */
      printf("n = %d\n", n);
      printf("%s\n", chead);
      printf("got %s.%s.%s npts=%d\n", shead->knetwk, shead->kstnm, shead->kcmpnm, (n - 632)/4);
    
    }
    if(n < 0) {
      printf("Error: %s\n", chead);
    }
    else if (n==0) {
      printf("no more files\n");
      printf("%s\n", chead);
    }
  }
  else
    printf("Error requesting data from server.\n");
  
  /* free the allocated data */
  free(chead);
  free(shead);
  free(*data);
  free(data);
}

void test_event() {
  int n, c;
  char **event;
  char ***event_ptr;
  STPEVENT **sevent;
  STPEVENT ***struct_ptr;

  /* request an event */
  n = stp_req_event("event -t0 2003/01/01 2003/01/04");
  
  /* if there is no error requesting the event .. */
  if (n != -1) {

    /* allocate space */
    event_ptr = (char ***) malloc(sizeof(char **));
    event_ptr[0] = NULL;
    struct_ptr = (STPEVENT ***) malloc(sizeof(STPEVENT **));

    /* get the events */
    n = stp_get_event(event_ptr, struct_ptr);

    /* if there was an error print it, free data, and exit */
    if (n <= 0) {
      
      n = -n; // change n back to a positive number
      if(event_ptr != NULL) {
	event = *event_ptr;
	if (event != NULL && event[0] != NULL) {
	  printf("%s", event[0]);
	}
	for (c=0; c<=n; c++) {
	  free(event[c]);
	}
	free(event);
	free(event_ptr);
      }

      if (struct_ptr != NULL) {
	sevent = *struct_ptr;
	
	for (c=0; c<=n; c++) {
	  free(sevent[c]);
	}
	free(sevent);
	free(struct_ptr);
      }

      exit(1);
    }

    /* make useful shortcut variables */
    if(event_ptr != NULL) {
      event = *event_ptr;
    }
    if (struct_ptr != NULL) {
      sevent = *struct_ptr;
    }

    /* print the event information */
    if (event != NULL) {
      for (c=0; c<=n; c++) {
	printf("%s", event[c]);
	if (sevent[c] != NULL) {
	  printf("eventid is %d\n", sevent[c]->eventid);
	  printf("type is %s\n", sevent[c]->type);
	  printf("datetime is %s\n", sevent[c]->datetime);
	  printf("lat is %f\n", sevent[c]->lat);
	  printf("long is %f\n", sevent[c]->lon);
	  printf("depth is %f\n", sevent[c]->depth);
	  printf("mag is %f\n", sevent[c]->mag);
	  printf("magtype is %c\n", sevent[c]->magtype);
	  printf("qual is %f\n", sevent[c]->qual);
	}
      }
      printf("\n");

      /* free the event information */
      for (c=0; c<=n; c++) {
	free(event[c]);
	free(sevent[c]);
      }
      free(event);
      free(sevent);
      
      
      free(event_ptr);
      free(struct_ptr);
    }
    
  }
  /* if the request failed */
  else
    printf("Error requsting event from server\n");
  
}

void test_altloc() {
  int n, c;
  char **altloc;
  char ***altloc_ptr;
  STPALTLOC **saltloc;
  STPALTLOC ***struct_ptr;

  /* request an altloc */
  n = stp_req_altloc("altloc -t0 2003/01/01 2003/01/04");
  
  /* if there is no error requesting the altloc .. */
  if (n != -1) {

    /* allocate space */
    altloc_ptr = (char ***) malloc(sizeof(char **));
    altloc_ptr[0] = NULL;
    struct_ptr = (STPALTLOC ***) malloc(sizeof(STPALTLOC **));

    /* get the altlocs */
    n = stp_get_altloc(altloc_ptr, struct_ptr);

    /* if there was an error print it, free data, and exit */
    if (n <= 0) {
      
      n = -n; // change n back to a positive number
      if(altloc_ptr != NULL) {
	altloc = *altloc_ptr;
	if (altloc != NULL && altloc[0] != NULL) {
	  printf("%s", altloc[0]);
	}
	for (c=0; c<=n; c++) {
	  free(altloc[c]);
	}
	free(altloc);
	free(altloc_ptr);
      }

      if (struct_ptr != NULL) {
	saltloc = *struct_ptr;
	
	for (c=0; c<=n; c++) {
	  free(saltloc[c]);
	}
	free(saltloc);
	free(struct_ptr);
      }

      exit(1);
    }

    /* make useful shortcut variables */
    if(altloc_ptr != NULL) {
      altloc = *altloc_ptr;
    }
    if (struct_ptr != NULL) {
      saltloc = *struct_ptr;
    }

    /* print the altloc information */
    if (altloc != NULL) {
      for (c=0; c<=n; c++) {
	printf("%s", altloc[c]);
	if (saltloc[c] != NULL) {
	  printf("eventid is %d\n", saltloc[c]->eventid);
	  printf("datetime is %s\n", saltloc[c]->datetime);
	  printf("lat is %f\n", saltloc[c]->lat);
	  printf("long is %f\n", saltloc[c]->lon);
	  printf("depth is %f\n", saltloc[c]->depth);
	  printf("mag is %f\n", saltloc[c]->mag);
	  printf("magtype is %c\n", saltloc[c]->magtype);
	  printf("nop is %d\n", saltloc[c]->nop);
	  printf("nos is %d\n", saltloc[c]->nos);
	  printf("totps is %d\n", saltloc[c]->totps);
	  printf("rms is %f\n", saltloc[c]->rms);
	  printf("err_z is %f\n", saltloc[c]->err_z);
	  printf("err_ew is %f\n", saltloc[c]->err_ew);
	  printf("err_ns is %f\n", saltloc[c]->err_ns);	 
	  printf("source is %s\n", saltloc[c]->source);
	}
      }
      printf("\n");

      /* free the altloc information */
      for (c=0; c<=n; c++) {
	free(altloc[c]);
	free(saltloc[c]);
      }
      free(altloc);
      free(saltloc);
      
      
      free(altloc_ptr);
      free(struct_ptr);
    }
    
  }
  /* if the request failed */
  else
    printf("Error requsting altloc from server\n");
  
}

void test_sta() {
  int n, c;
  char **sta;
  char ***sta_ptr;
  STPSTA **ssta;
  STPSTA ***struct_ptr;

  /* request station information */
  n = stp_req_station("sta 1999 2000\0");
  
  if (n != -1) {
    /* allocate space */
    sta_ptr = (char ***) malloc(sizeof(char **));
    sta_ptr[0] = NULL;
    struct_ptr = (STPSTA ***) malloc(sizeof(STPSTA **));
    
    /* get station information */
    n = stp_get_station(sta_ptr, struct_ptr);
 
    if (n <= 0) {
    
      n = -n; // make n positive
     
      if(sta_ptr != NULL) {
	sta = *sta_ptr;
	if (sta != NULL && sta[0] != NULL) {
	  printf("%s", sta[0]);
	}
	for (c=0; c<=n; c++) {
	  free(sta[c]);
	}
	free(sta);
	free(sta_ptr);
      }
      
      if (struct_ptr != NULL) {
	ssta = *struct_ptr;
	
	for (c=0; c<=n; c++) {
	  free(ssta[c]);
	}
	free(ssta);
	free(struct_ptr);
      }
      exit(1);
    }
    

    /* setup shortcut variables */
    if(sta_ptr != NULL) {
    sta = *sta_ptr;
    }
    if (struct_ptr != NULL) {
      ssta = *struct_ptr;
    }
    
    /* print out station information */
    if (sta != NULL) {
      for (c=0; c<=n; c++) {
	printf("%s", sta[c]);
	if (ssta[c] != NULL) {
	  printf("net is %s\n", ssta[c]->net);
	  printf("sta is %s\n", ssta[c]->sta);
	  printf("lat is %f\n", ssta[c]->lat);
	  printf("long is %f\n", ssta[c]->lon);
	  printf("elevation is %f\n", ssta[c]->elev);
	  
	}
      }
      printf("\n");
      
      /* free up allocated space */
      for (c=0; c<=n; c++) {
	free(sta[c]);
	free(ssta[c]);
      }
      free(sta);
      free(ssta);
      
      free(sta_ptr);
      free(struct_ptr);
    }
  }
  else
    printf("Error requesting stations from the server\n");
}

void test_phase() {
  int n, m, i, j;
  char **strinfo;
  STRINGPHASE ***strphase;
  STRINGPHASE **phase;
  STPPHASE ***stpphase;
  STPPHASE **stppha;
  PHAINFO *pha;
  
  /* request phases */
  n = stp_req_phase("phase -t0 -6h +6h");

  /* if there is no error requesting ... */
  if (n != -1) {

    /* allocate space */
    strinfo = (char **) malloc(sizeof(char *));
    if (strinfo == NULL) {
      printf("Allocation failed.\n");
      return;
    }
    strphase = (STRINGPHASE ***)malloc(sizeof(STRINGPHASE **));
    if (strphase == NULL) {
      printf("Allocation failed.\n");
      return;
    }
    stpphase = (STPPHASE ***) malloc(sizeof(STPPHASE **));
    if (stpphase == NULL) {
      printf("Allocation failed.\n");
      return;
    }
    
     // get the phase, you can have strphase = NULL and/or stpphase = NULL
     n = stp_get_phase(strinfo, strphase, stpphase);

    /* if an error occured, print it and exit */
    if (n <= 0) {
      printf("%s", *strinfo);
      
      n = -n; // make n positive
      /* free the strings */
      if (strphase != NULL && *strphase != NULL) {
	phase = *strphase;
	for (i = 0; i<n; i++) {
	  free(phase[i]->evtinfo);
	  m = phase[i]->numphastr;
	  for (j=0; j<m; j++) {
	    free(phase[i]->phastr[j]);
	  }
	}
      }

      /* free the structs */
      if(stpphase != NULL && *stpphase != NULL) {
	stppha = *stpphase;
	for (i = 0; i<n; i++) {
	  
	  free(stppha[i]->evtinfo);
	  m = stppha[i]->numphainfo;
	  for (j = 0; j<m; j++){
	    free(stppha[i]->phainfo[j]);
	  }
	}
      }
    
      /* free allocated space */
      free(strinfo);
      free(strphase);
      free(stpphase);  

      exit(1);
    }
    
    /* print out the strings */
    phase = *strphase;
    for (i = 0; i<n; i++) {
      printf("\n%s", phase[i]->evtinfo);
      m = phase[i]->numphastr;
      for (j=0; j<m; j++) {
	printf("%s", phase[i]->phastr[j]);
      }
    }
    
    /* free the strings */
    for (i = 0; i<n; i++) {
      free(phase[i]->evtinfo);
      m = phase[i]->numphastr;
      for (j=0; j<m; j++) {
	free(phase[i]->phastr[j]);
      }
    }
    
    /* print out the structs */
    stppha = *stpphase;
    for (i = 0; i<n; i++) {
      printf("eventid is %d\n", stppha[i]->evtinfo->eventid);
      printf("type is %s\n", stppha[i]->evtinfo->type);
      printf("datetime is %s\n", stppha[i]->evtinfo->datetime);
      printf("lat is %f\n", stppha[i]->evtinfo->lat);
      printf("long is %f\n", stppha[i]->evtinfo->lon);
      printf("depth is %f\n", stppha[i]->evtinfo->depth);
      printf("mag is %f\n", stppha[i]->evtinfo->mag);
      printf("magtype is %c\n", stppha[i]->evtinfo->magtype);
      printf("qual is %f\n", stppha[i]->evtinfo->qual);
      
      m = stppha[i]->numphainfo;
      for (j = 0; j<m; j++){
	pha = stppha[i]->phainfo[j];
	printf("%s %s %s %f %f %f %c %s %c %f %f %f\n", pha->net, pha->sta, pha->chan, pha->lat, pha->lon, pha->elev, pha->phase, pha->motion1, pha->soqual, pha->pickqual, pha->epidist, pha->time) ;
      }
    }
    
    /* free the structs */
    for (i = 0; i<n; i++) {
    
      free(stppha[i]->evtinfo);
      m = stppha[i]->numphainfo;
      for (j = 0; j<m; j++){
	free(stppha[i]->phainfo[j]);
      }
    }
    
    /* free allocated space */
    free(strinfo);
    free(*strphase);
    phase = NULL;
    free(*stpphase);
    stppha = NULL;
    free(strphase);
    free(stpphase);
  }
  else
    printf("Error requesting phases from server.\n");

  return;
}
